import java.io.*;

public class Vigenere {

	public static String filename;
	public static String operation;
	public static String keyword;
	public static String outputFilename;
	protected static int fileChar;  // used for storing one character at a time from the file read from the disk
	protected static boolean thereIsAFile = true;  // used for checking the existence of the file on the disk

	// The following 4 variables are used for storing the answer of the user to the question 
	//of whether or not they wish to see the file content on screen
	public static String showInputFileConsole;  
	public static String showOutputFileConsole;
	public static boolean showInputFile;
	public static boolean showOutputFile;
	
	//The following 3 objects are used for setting up the mechanism of reading the file from disk
	protected static FileInputStream fileInStream;
	protected static InputStreamReader inputReader;
	protected static BufferedReader bufReader;

	//These objects have been automatically created by Omondo. Their existence is rather unnecessary.
	private Ciphering ciphering;
	private Deciphering deciphering;

	//The class constructor
	public Vigenere() {
		super();
	}
	public static void main(String[] args) {
		try{
			//The program takes 3 arguments and saves them in the following 3 variables.
			filename = args[0];
			operation = args[1];
			keyword = args[2];
			
			//This function checks the validity of the operation and exitence of the file on the disk.
			inputValidation(filename, operation);
			
			//The main functionality of the program and interaction with the user are implemented in this function. See below for details.
			process(operation);			
			}
		//The following exceptions have been handled as seen below. Basically, they inform the user of the error encountered.
		catch (FileNotFoundException e1) {System.out.println("There has been a disk related error.");}
		catch (IOException e2) {System.out.println("There has been an input/output error.");}
		catch (ArrayIndexOutOfBoundsException e3){System.out.println("There has been a problem in the " +
				"way the arguments have been entered. Please use the following format:");
		System.out.println("java Vigenere filename.txt operation keyword");
		System.out.println("In which the operation can be either \"ciphering\" or \"deciphering\".");}
	}

	public static void inputValidation(String filename, String operation)throws FileNotFoundException,IOException{
		try{
			//Here, we check the operation validity.
			if (operation.equals("ciphering") || operation.equals("deciphering"))
				System.out.println("The operation is valid!");
			else{
				System.out.println("The operation is invalid!");
				System.out.println("The operation can be either \"ciphering\" or \"deciphering\".");
				System.out.println("Please use the following format: java Vigenere filename.txt operation keyword");
			}
			//Here, we try to open the file, filename of which is provided by user. The file is immediately closed. File existence checking.
			FileInputStream temp = new FileInputStream(filename);
			temp.close();
			System.out.println("The input file is valid!");
			}
		catch (FileNotFoundException e1) {
			System.out.println("There is no such a file on the disk!");
			// If the file doesn't exist, this value is set accordingly. 
			//procees() function reads this value before execution.
			thereIsAFile = false;   
			}
	}
	public static void process(String operation)throws FileNotFoundException,IOException{
		
		if (thereIsAFile == false) {return;}
		
		//Here, we set up the mechanism by which we read the user input to the question of whether or not they want to see the file content.
		BufferedReader console = new BufferedReader(new InputStreamReader(System.in));

		//If the operation is ciphering, we move to this block of code which first checks to see if the user wants to see the content of 
		//file before proceeding. Then, the file is ciphered. At the end, the user is asked to see whether or not they like to see the
		//content of the ciphered file.
		if (operation.equals("ciphering")){
			try {
				System.out.print("Would you like to see the content of the input file on the screen? [y/n]");
				 showInputFileConsole= console.readLine();
				 if (showInputFileConsole.equals("y") || showInputFileConsole.equals("Y")){
					showInputFile = true;
					System.out.println("\nHere is the content of the input file:");
					//The following function shows the content of the file. It is defined below.
					showFileContent(filename);
				 }
				 else if (showInputFileConsole.equals("n") || showInputFileConsole.equals("N"))
					 showInputFile = false;
				 else
					 System.out.println("Please enter either \"y\" or \"n\"");
						 
			}catch (IOException e) {System.out.println("Please enter either \"y\" or \"n\""); }
			
			//Here, an instance of the class Ciphering is created and the filename and the operation are passed to its constructor.
			Ciphering ciphering = new Ciphering(filename , keyword);

			//The cipher() function of the ciphering object is called to begin the ciphering operation.
			ciphering.cipher();
			
			System.out.print("Would you like to see the content of the ciphered text " +
				"on the screen? [y/n]");
			showOutputFileConsole= console.readLine();
			 if (showOutputFileConsole.equals("y") || showOutputFileConsole.equals("Y")){
				showOutputFile = true;
				System.out.println("\nHere is the content of the ciphered file:");
				showFileContent(ciphering.outputFilename);
			 }
			 else if (showOutputFileConsole.equals("n") || showOutputFileConsole.equals("N"))
				 showOutputFile = false;
			 else
				 System.out.println("Please enter either \"y\" or \"n\"");
		}
			
		//If the intended operation is Deciphering, we move to this block of code. The sequence of the actions is exactly 
		//like those of the ciphering operation described above.
		else if (operation.equals("deciphering")){
			try {
				System.out.print("Would you like to see the content of the input file on the screen? [y/n]");
				 showInputFileConsole= console.readLine();
				 if (showInputFileConsole.equals("y") || showInputFileConsole.equals("Y")){
					showInputFile = true;
					System.out.println("\nHere is the content of the input file:");
					showFileContent(filename);
				 }
				 else if (showInputFileConsole.equals("n") || showInputFileConsole.equals("N"))
					 showInputFile = false;
				 else
					 System.out.println("Please enter either \"y\" or \"n\"");
						 
			}catch (IOException e) {System.out.println("Please enter either \"y\" or \"n\""); }
			
			Deciphering deciphering = new Deciphering(filename , keyword);

			deciphering.decipher();
			
			System.out.print("Would you like to see the content of the deciphered text " +
				"on the screen? [y/n]");
			showOutputFileConsole= console.readLine();
			 if (showOutputFileConsole.equals("y") || showOutputFileConsole.equals("Y")){
				showOutputFile = true;
				System.out.println("\nHere is the content of the deciphered file:");
				showFileContent(deciphering.outputFilename);
			 }
			 else if (showOutputFileConsole.equals("n") || showOutputFileConsole.equals("N"))
				 showOutputFile = false;
			 else
				 System.out.println("Please enter either \"y\" or \"n\"");
		}
	}
	
	public static void showFileContent(String filename)throws FileNotFoundException,IOException{

		//The following 3 objects are created for setting up the mechanism by which we read the file. 
		fileInStream =new FileInputStream(filename);
		inputReader = new InputStreamReader(fileInStream);
		bufReader = new BufferedReader(inputReader);

		System.out.println("***************************************");
		 
		while( (fileChar = bufReader.read())!= -1){
			 System.out.print((char)fileChar);
			 };

		System.out.println("\n***************************************");

		//After finishing reading the file, we delete the set up mechanism.
		bufReader.close();
		inputReader.close();
		fileInStream.close();
	}

	/* Automatically created by the Omondo plug-in. These are not used in the program.
	These has to be kept in order for the UML model to show the correct associations. */

	public Ciphering getCiphering() {
		return ciphering;
	}
	public void setCiphering(Ciphering ciphering) {
		this.ciphering = ciphering;
	}

	public Deciphering getDeciphering() {
		return deciphering;
	}
	public void setDeciphering(Deciphering deciphering) {
		this.deciphering = deciphering;
	}
}
